% Basic MATLAB tutorial
% Predrag Radivojac
% Department of Computer Science and Informatics
% Indiana University
% Bloomington, Indiana
% predrag@indiana.edu

% Tutorial is available at
% www.informatics.indiana.edu/predrag

% Last update August 2014

%% ------------------------------------------------------------------------
% 1. MATLAB Basics
%--------------------------------------------------------------------------

% MATLAB is an interpreting programming language that supports all basic 
% operations and functions like other programming languages (e.g. if-else 
% structure, for loop, while loop, case-switch structure, break, try, 
% catch, return, ...). But let us start with real basics...

% check the version of MATLAB
version

% check the version of MATLAB, with all toolboxes listed
ver

% check your working directory
pwd

% check the operating system and print string on the screen (no, we are not
% starting with "Hello World"); observe that we will use single quotes for
% strings
if ispc == 1
    fprintf(1, '\nThis is a Windows machine!\n\n');
elseif isunix == 1
    fprintf(1, '\nThis is a Unix machine!\n\n');
else
    % we should not be in here, but this is how you may handle termination
    warning('We are using an unknown platform!');
    error('We are using an unknown platform, good bye!');
end

% or we can do it this way (type "help computer" to get more details)
computer

% we will now include our subfolders to the path, so we can see functions
p = genpath(pwd); % find all folders in our current folder
addpath(p); % add them to the path (addpath(genpath(pwd)) would do too)

% delete all variables from the workspace (here, this is same as 'clear')
% 'clear all' will not remove the new folders from the path; it also won't
% close open files if there are any
clear all

% clear screen
clc

% define variable 'a'
% 'a' will be displayed on the screen; semi-colon will stop printouts; you
% do not have to define type of a variable nor worry about memory 
% allocation.
a = 10

% using 'if' structures
if (a == 4)        % parentheses are not required
   b = a ^ 2       % 'b equals a squared', b is printed on the screen
elseif a ~= 10     % this means 'if a is not equal to 10'
   b = a           % then 'b equals a'
else
   b = -a
end                % there is only one 'end' for the whole 'if' structure

% using a 'for loop': start with 1, step is 2, end if i >= 10
for i = 1 : 2 : 10
	i   
end

% using a 'while loop': it is sometimes preferred to a for loop
i = 1;
while i < 10
    i = i + 2
end

% MATLAB supports help system through its help window, or through help 
% statement from the command prompt (e.g. help case)
help case

% There is also a 'lookfor' function, when we are not sure what is the 
% exact name of the statement or the function. Lookfor searches for 
% specifiec word through description of all available MATLAB functions e.g. 
% 'lookfor write' gives all functions that contains the word write in their
% description.

% Unlike other languages it DOES NOT make executable code (a special 
% toolbox is needed for it). Instead, it interprets comands of functions 
% written in the script or inside a function.

% Function has a specific syntax and should be called to be executed.
% It can have more than one input and more than one output argument.

a = 5; % 'a' is not displayed
b = 4  % 'b' is displayed

% 'operations' is a function that takes two input variables and returns two
% output variables; these two are displayed since no semi-colon is after
% 'operations'
[sum_ab, product_ab] = operations(a, b)
pause(3) % go to sleep for 3 seconds

% MATLAB passes variables into functions 'by value' which means it copies
% variables into local variables with the same names. However, newer
% versions of MATLAB perform copy-on-write, which means that the variables
% won't be copied unless you modify them inside the function. To avoid this 
% situation, you can define variables to be 'global' so that you can change
% them within a function. An example of this is below.

global ag bg
ag = a;
bg = b;

fprintf(1, '\nBefore: a = %d, b = %d, ag = %d, bg = %d\n', a, b, ag, bg);
change_input_variables(a, b); % this function does not return anything
fprintf(1, 'After : a = %d, b = %d, ag = %d, bg = %d\n\n', a, b, ag, bg);

% Did you notice that a and b had a local copy within the function, but
% then returned to their regular values once we exited the function?

%% ------------------------------------------------------------------------
% 2. Vector and Matrix Operations
%--------------------------------------------------------------------------

A = [ 1  2  3 
      4  5  6
      7  8  9
     10 11 12]
   
B = [ 1,  3,  5,
      7, 11, 13,
     17, 19, 23,
     29, 31, 37]
   
% more compact way to initialize a matrix; observe the role of semi-colon   
C = [1 2 3 4; 5 6 7 8; 9 10 11 12]

% MATLAB is very powerful in matrix computations
% do not use 'for' loops for matrix calculations unless you really have to
% see some matrix operations below
D = A + B

E = A * C

F = C * B

% inverse of the square root of matrix F
F1 = inv(sqrt(F))

% when we use of a dot "." BEFORE a command it leads to per-component
% operations
A
B
G = B ./ A
G_transpose = G'
G_transpose = transpose(G)

% go to sleep until interrupted (you need to press any key in MATLAB's main
% window)
pause

% dimensions of the matrix 'A'
[num_rows, num_cols] = size(A)

% or only the number or rows (first dimension)
num_rows = size(A, 1)

% or only the number or columns (second dimension)
num_cols = size(A, 2)
 
% colon operator is very important in MATLAB; e.g. let's extract columns 
% from a matrix
A
for i = 1 : size(A, 2)
  A_column = A(:, i)
end

% let's now extract rows 
A
for i = 1 : size(A, 1)
  A_row = A(i, :)
end  

% special matrix functions
diagonal_matrix = diag([1 2 3 4])
all_zeros = zeros(3, 4)
all_ones = ones(4, 2)
identity_matrix = eye(4)

% let's now define two column vectos
v1 = [1 2 3 4 5]';
v2 = [5 4 3 2 1]';

% first, let's calculate a dot (scalar) product
v3 = v1' * v2

% now, let's calculate a vector whose components are squares of v1
v4 = v1 .^ 2

% ... and another vector whose components are square roots of v1
v5 = v1 .^ 0.5

% finally, let's calculate Euclidean distance between v4 and v5
d = sqrt((v4 - v5)' * (v4 - v5))

%% ------------------------------------------------------------------------
% 3. Basic Set Operations
%--------------------------------------------------------------------------

% define two sets, as vectors
a = [1 2 3 4 5 6 7]
b = [2 4 6 8 10 12]

% find intersection of the two sets (will be a sorted array)
c = intersect(a, b)

% find union of the two (will be a sorted array)
c = union(a, b)

% find difference of the two (note: it is a directional operation)
c = setdiff(a, b)

% let's demonstrate that sets cannot have duplicate elements
a = [1 2 3 3]
b = [2 2 3 4]

% all duplicate elements will disappear after a set operation
c = union(a, b)

% check if 1 is a member of set c
ismember(1, c)

% we can make a set out of a vector using 'unique'
a = unique(a)

%% ------------------------------------------------------------------------
% 4. Basic Operations on Strings
%--------------------------------------------------------------------------

% define two arbitrary strings
s1 = 'WBuCDefg3'
s2 = 'AAAaAAA'

% concatenate two strings
s = [s1 s2]

% this works with matrices too
A
a1 = [9 8 7 6]'
A1 = [A a1]
a1 = [0 0 0 0]
A1 = [A1; a1]

% back to strings, let's find the beginning of the substring 'AAA' in s
pos = findstr('AAA', s)

% 'findstr' will find the position of the shorter in the longer of the two
% strings; use function 'strfind' to strictly look for 'AAA' in s.


% let's form a cell array of strings... (note curly braces for cell arrays)
S{1} = s1
S{2} = s2
S{3} = s
% ... and find all that start with 'AAA'
pos = strmatch('AAA', S)
% ... now find only exact matches (shouldn't be any!)
pos = strmatch('AAA', S, 'exact')

% another way to define a cell array of strings
S = {'12345', 'abcde', 'university', 'matlab', 'matrix'}

% let's find a position of the string 'matlab' in S
pos = strmatch('matlab', S, 'exact')

% compare two strings, a standard operation
strcmp(s1, s2)

% check if a variable is a string
isstr(s1) % this IS a string
isstr(S)  % this is NOT a string (it is a cell array of strings)

% replace all occurrences of AA with BB in s2 (Note: overlaping strings can
% cause string expansion; here s3 will be longer than s2)
s3 = strrep(s2, 'AA', 'BB')

% replacing AA with BB and then BB with AA may not give you the original 
% string 
s4 = strrep(s3, 'BB', 'AA')

% MATLAB also works with regular expressions, here is an example from its
% help; type "help regexprep" for details of usage
str = 'I walk up, they walked up, we are walking up, she walks.'
pat = 'walk(\w*) up'
regexprep(str, pat, 'ascend$1')

% now convert all letters to capital ('lower' would invoke lower case)
upper(regexprep(str, pat, 'ascend$1'))

%% ------------------------------------------------------------------------
% 5. 'find' function and logical indexing
%--------------------------------------------------------------------------

a = [1 2 3 4 5 6 7];
b = [2 4 6 8 10 12];
c = union(a, b)

% find all indices where the elements of c are greater than 3
pos = find(c > 3)

% let's do a more complicated one
pos = find(c > 3 & mod(c, 2) == 0) % greater than 3 and even ('&' is a logical AND)

% let's digress: type "help ^" to see all arithmetic, logical, bitwise, 
% relational, and set operations. It will also give you special characters
% in MATLAB

% let's change all elements found before
% note that we are indexing using vector 'pos'; this is an important
% feature!!! How would you index in C/C++ using using vectors?
c(pos) = 999

% instrad of logical AND use logical OR
pos = find(c > 3 | mod(c, 2) == 0) % greater than 3 or even
c(pos) = 555

% let's check how this works with a matrix
A = [1 4 7; 2 5 8; 3 6 9]
pos = find(A > 4)
A(pos) = 100

% to get indices of the positions in the matrix, we need to modify previous
% statement
[i j] = find(A > 4);
[i j]

% now, let's do it with logical indexing
c = union(a, b)
pos = find(c > 3 & mod(c, 2) == 0);

% logical indexing gives you an array of the same dimensions where each
% position satisfies the condition
d = (c > 3 & mod(c, 2) == 0)

% let's try to get our pos variable through this
pos2 = find(d > 0)

% we can modify indexing to achieve the same thing
c(d) = 555

% let's see how it affects matrices
B = (A > 4)

% we will modify only the posistions greater than 4
A(B) = 222

%% ------------------------------------------------------------------------
% 6. Input/Output, System Calls
%--------------------------------------------------------------------------

% clear screen
clc

% check what variable we have in memory
who

% check what variable we have in memory (more detailed)
whos

% check if matrix A is in the workspace
whos A

% save specified variables into a specified file in ascii format
E
save ascii_matrix.txt E -ascii 

% save specified variables into a specified file in the binary 
% (MATLAB-specific) format default extension is *.mat
save bin_matrices A B C

% to save ALL variables, write only a file name. All variables in scope 
% will be saved
save binary_all

clear A
whos       % 'A' is not listed (if it is we should contact MathWorks!!!)

clear all  % delete all variables in scope
clc        % clear screen
whos       % you should see an empty list!

% load variable 'A' from a *.mat file
load binary_all A
whos

% load all variables from a *.mat file
load binary_all 
whos

% load from an ASCII file
E_matrix = load('ascii_matrix.txt')
% or 
load ascii_matrix.txt
E_matrix = ascii_matrix
E

% check if we are on Windows or a Unix system to copy a file
% we will ececute an external command (symbol "!" indicates OS command)
if ispc == 1
    !copy ascii_matrix.txt copy_of_ascii_matrix.txt
else
    !cp ascii_matrix.txt copy_of_ascii_matrix.txt
end

% delete a file through MATLAB
delete('copy_of_ascii_matrix.txt');

% now, make it again using command 'system' (you can use 'dos' if you are
% using a Windows platform and 'unix' for a Unix platform.
if ispc == 1
    system('copy ascii_matrix.txt copy_of_ascii_matrix.txt');
else
    system('cp ascii_matrix.txt copy_of_ascii_matrix.txt');
end

% eval command, very useful in many cases (here just to delete a file, 
% system command again)
file_to_delete = 'copy_of_ascii_matrix.txt';
if ispc == 1
    eval(['! del ' file_to_delete]);
else 
    eval(['! rm ' file_to_delete]);
end

% if you don't want to see operating system displays use 'system' command
% 's' will be the status and 'w' will be OS output
if ispc == 1
    [s, w] = system('dir');
else 
    [s, w] = system('ls');
end
display(w); % see what's in the current directory

% external function add.pl is a Perl code. Here, we will try to add 3 and 5. 
% Note that you need to convert these numbers to strings because Perl 
% assumes strings as inputs. If Perl command is not working, type 'help 
% perl' for further directions on how to execute Perl under MATLAB.
s = perl('add.pl', num2str(3), num2str(5)); % s is a string

% the result after this command should be 10
str2num(s) + 2

% MATLAB has many good functions for reading and writing files; check
% functions 'fread' and 'fwrite' for example. Also, to read specific
% formats, such as Excel files you can use 'xlsread', or to read custom
% formatted text files you can type 'help textscan' to check this function.

% OK, let's provide one example of reading and writing binary files
clear 
clc

% create a vector of 15 numbers
x = [1 2 3 4 5 5 4 3 2 1 1 2 3 4 5];

% we will now write x to a binary file

% first, open a file for writing ('w') using binary ('b') format
fid = fopen('temp_binary_file.bin', 'wb');
if fid == -1
    error('Cannot open file');
end

% 'fid' is a file handle, it keeps track of the actual file opened on the
% disk; you cannot get these details from fid, but you can get a value that
% indicates whether the operation was completed successfully (e.g. -1 means
% it wasn't)

% write (save) vector x in double precision (64 bits = 8 bytes per element)
count = fwrite(fid, x, 'double'); % 'count' says how many numbers were read

% close file; the file should be of size 15 * 8 = 120 bytes
fclose(fid);

% we will now read this binary file into vector y

% first, open the file for reading ('r') given its binary format ('b')
fid = fopen('temp_binary_file.bin', 'rb');
if fid == -1
    error('Cannot open file');
end

% read (load) elements in double precision (64 bits = 8 bytes per element)
y = fread(fid, count, 'double'); % 'y' will be a column vector

% close file
fclose(fid);

% compare x and y, just to check whether they are the same
s = sum(x - y') % we transposed 'y'

% remove the files we don't need
delete('bin_matrices.mat');
delete('binary_all.mat');
delete('temp_binary_file.bin');

%% ------------------------------------------------------------------------
% 7. Graphics
%--------------------------------------------------------------------------

clear
clc

% define vector t (to indicate time)
t = 0 : 0.1 : 5
x = sin(pi * t) + 0.1 * t;
y = cos(pi * t) - 0.2 * t;
plot(t, x);
plot(t, y);
xlabel('Time');     % label x-axis
ylabel('Function'); % label y-axis
axis([0 10 -2 2]);  % change axis; e.g. x-axis from 0 to 10, y from -2 to 2

% plot y as a function of x
plot(x, y);       % plots 'y' as a function of 'x'
hold on           % enables plotting one figure over another
plot(x, y, 'r*'); % plots red stars instead of line

% get handle of current figure and close it
h = gcf;
close(h);

% play a bit more with x, y, and z
figure;                 % starts a new (empty) figure
z = 2 * t;
plot3(x, y, z, 'm-d');  % shows magenta diamonds in addition to line

% try a 3-D function (example from MATLAB's Demo)
z = peaks(25);                          
surfl(z);                             
shading interp;                       
colormap(pink);      
close(gcf); % close figure, in one line

% other nice functions for visualization: 'imagesc', 'meshgrid', ...
% use HELP for these commands

% for example, 'imagesc' is used to plot a matrix
E = load('ascii_matrix.txt')
imagesc(E)
close(gcf);

% clean the file
delete('ascii_matrix.txt');

% check functions 'imread', 'imwrite' to load/save an image into a matrix

%% ------------------------------------------------------------------------
% 8. Some Useful Functions, Approaching Data Analysis Stage
%--------------------------------------------------------------------------

A = [ 0    7     1     5     0
      1    7     4     6     1
      4    3     9     1     5
      8    9     4     5     0
      9    3     3     5     2
      0    1     4     9     3
      7    7     7     9     9
      8   10     7     5     3
      7    2     9     8     3
     10    4     8     7     4]

A1 = sort(A)       % SORT each column separately
A2 = sort(A, 2)    % SORT each row separately

% find all rows where the values in the second columen are greater than 5
A
pos = find(A(:, 2) > 5)
A_reduced = A(pos, :) % print these rows

% another way to do the same thing (in one line)
A_reduced = A(find(A(:, 2) > 5), :)

% the same thing, but with logical indexing
A_reduced = A(A(:, 2) > 5, :)


% let us now see some basic data analysis and visualization
A
sum(A)       % sumation performed over columns
sum(A, 1)    % same as above
sum(A, 2)    % sum of elements, row by row
sum(sum(A))  % sum all elements in the matrix

A
mean_A_col = mean(A)        % mean over columns
mean_A_row = mean(A, 2)     % mean over rows
std_A_col = std(A)          % standard devaition
std_A_row = std(A, [], 2)   % standard deviation ([] MUST be here - check help for details)
median_A_col = median(A, 1) % median
median_A_row = median(A, 2) % median

% generate a matrix of random numbers (uniform distribution)
B = rand(5000, 5);
hist(B(:, 1), 20);   % histogram of column 1 in 20 bins

% generate a matrix of random numbers (normal dustribution)
B = randn(5000, 5);
hist(B(:, 5), 30);   % histogram of column 5 in 30 bins

% change some properties of the histogram
h = findobj(gca, 'Type', 'patch');
set(h,'FaceColor','w','EdgeColor','k', 'LineWidth', 1);
xlabel('Number', 'fontsize', 14, 'fontweight', 'b', 'color', 'b');
ylabel('Count', 'fontsize', 14, 'fontweight', 'b', 'color', 'r');
close(gcf);

% a more advanced function to superiimpose two histograms
superimpose_histograms(randn(5000, 1), randn(5000, 1) + 3);
% then, we will export it into eps, png and tiff formats
export_figure(gcf);
close(gcf);
delete('myfig.*');

% back to the 'sort' function for the moment
a = [4 9 24 8 -13 9 3]
% 'x' will be the sorted array, 'y' are the indices of 'x' in 'a'
[x, y] = sort(a)
% do you need it in the descending order? Note the absence of comma.
[x y] = sort(a, 'descend')
% do you only want y, but not x? Use a tilda, but then a comma is mandatory
[~, y] = sort(a, 'descend')


%% ------------------------------------------------------------------------
% 9. Structures in MATLAB
%--------------------------------------------------------------------------

% clear workspace and screen
clear
clc

% let's show an example how structures can be used to group data
% we will define two variables, 'x' and 'y', and each of them will be a 
% structure

x.name = 'Alice'
x.title = 'Professor'
x.songs = {'Another one bites the dust'  'November rain'}; 
% field 'songs' is a cell array of strings; note that no commas are needed 
% to initialize a cell array of strings (same holds for vectors)

y.name = 'Bob';
y.title = 'Student';
y.songs = {'Oops! ...I Did It Again'  'Beautiful' ...
    'Baby don''t forget my number'}; % observe three points above

% connect x and y into one array of structures; note that [] can be used
% for variables, for strings or for structures
z = [x y]

z(2).songs

% get the field title from structure y
getfield(z(2), 'title')

% check whether certain field is there
isfield(z, 'bands')

% let's see how one can sort the field names; Note: z won't be changed
z = orderfields(z)

% let's get all the fieldnames for variable z and put them in cell array s
s = fieldnames(z)

% let's remove field 'name' from the array of structures
rmfield(z, s{1})

% let's now show how fields can be used dynamically, on a slightly
% different example; let's assume the class number is B490
B490.Anna.quizscores = [13 18 18]
B490.Anna.hwscores = [40 100 72 56]
B490.Anna.midterm = 80

B490.Nick.quizscores = [20 14 17]
B490.Nick.hwscores = [40 95 80 0]
B490.Nick.midterm = 77

class_score(B490, 'Anna')
class_score(B490, 'Nick')

% or we can do it this way
s = fieldnames(B490);
for i = 1 : length(s)
    fprintf(1, 'Score for %s is %.1f\n', s{i}, class_score(B490, s{i}));
end

%% ------------------------------------------------------------------------
% 10. Function Handles
%--------------------------------------------------------------------------

% let's define function handle that squares the argument variable
% '@' denotes a function handle, and x in the parentheses indicates an 
% input variable
sqr = (@(x) x .^ 2);

% let's execute sqr function using vector [1 2 3] as input
sqr([1 2 3])

% let's now define a dot product function handle...
dot_product = (@(x, y) x' * y);

% ...and use it
a = [1 2 3]';
b = [4 5 6]';
dot_product(a, b)

% let's now plot some functions defined by their handles
fun1 = (@(x) 2 * x .^ 2 + 3 * x + 4);
fplot(fun1, [-5 5])

fun2 = (@(x) -2 * x .^ 2 + 3 * x);
fplot(fun2, [-5 5])
close(gcf)

% let's define Mahnattan distance function handle
distance_M = (@(x, y) sum(abs(x - y)));
% let's define Euclidean distance function handle
distance_E = (@(x, y) sqrt(sum((x - y) .* (x - y))));

% let's calculate distances by passing function handles as arguments
calculate_distance([1 2 3]', [4 5 6]', distance_E)
calculate_distance([1 2 3]', [4 5 6]', distance_M)

% Finally, let's show how to convert strings into function handles
s = 'distance_Lmax'; 
f = str2func(s);
calculate_distance([1 2 3]', [4 5 6]', f)
% note: function 'distance_Lmax' must exist as a .m file

%% ------------------------------------------------------------------------
% 11. Using the Internet
% You should be on-line for this part to work; 
% There may be some firewall issues as well;
%--------------------------------------------------------------------------

clear
clc

% MATLAB allows you to read data directly from the Internet; for example
% one can read a web site by using function 'urlread'

s = urlread('http://en.wikipedia.org')

% you can also directly communicate with FTP sites; for example, we will 
% load the Boston Housing data from UC Irvine's Machine Learning repository
% Boston Housing data set is stored as a matrix (in text format)

% open FTP connection
handle = ftp('ftp.ics.uci.edu');

% change directory to housing
cd(handle,'pub/machine-learning-databases/housing/');

% download data to file (you may need to disable Windows firewall here!)
filename = mget(handle,'housing.data');

% load the matrix from file
M = load(filename{1})

% close FTP connection
close(handle);

% remove the file
delete('housing.data');

% remove variables from memory and clear screen
clear
clc

%% ------------------------------------------------------------------------
% The End
%--------------------------------------------------------------------------
